#!/usr/bin/python

#/*
# * pyKino - Kino
# *
# * Copyright (C) 2004 Josef "Jeff" Sipek <jeffpc@optonline.net>
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License version 2 as
# * published by the Free Software Foundation.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# *
# * @(#) pykino.py 1.4 04/11/03 22:34:43
# */

version = "0.11"

try:
	import sys
	import threading
	import pygame
	import random
	import math
	
	import pykino_colors
	import pykino_utils
	import config
except ImportError, err:
	print "Couldn't load module %s" % (err)
	sys.exit()

size = config.width, config.height
minutedelay = config.delay
sounds = config.sound

def main():
	""" Main fn to run the main thread """
	
	# print configuration
	print_config()
	
	# Init
	speed = config.speed
	
	pygame.init()
	screen = pygame.display.set_mode(size)
	pygame.display.set_caption('pyKino (v. ' + version + ')')
	
	# background music
	if (sounds):
		pygame.mixer.init()

	# Set background
	background = pygame.Surface(screen.get_size()).convert()
	background.fill(pykino_colors.black)
	
	(splash_image, splash_rect) = pykino_utils.load_png('data/image/splash.png')
	background.blit(splash_image, (0, 0))

	#blit!
	screen.blit(background, (0, 0))
	pygame.display.flip()
	
	random.seed()
	
	playgame = 0

	while 1:
		for event in pygame.event.get():
			if event.type == pygame.QUIT:
				sys.exit()
			if (event.type == pygame.KEYDOWN) and (not playgame):
				playgame = 1
		
		if (playgame):
			print "Running a new game..."
			speed = game(screen, speed)
			print "Game over!"

def calcx(num):
	return (num % 8) * 100 + 50

def calcy(num):
	return (num / 8) * 55 + 27

def game(screen, speed):
	""" Main fn to run the game """
	
	# Set background
	background = pygame.Surface(screen.get_size()).convert()
	background.fill(pykino_colors.black)
	
	numbers = pygame.Surface(screen.get_size()).convert()
	font = pygame.font.Font(None, 36)
	for x in range(0, 80):
		text = font.render(str(x+1), 1, pykino_colors.red)
		textpos = text.get_rect()
		textpos.centerx = calcx(x)
		textpos.centery = calcy(x)
		background.blit(text, textpos)
	
	#blit!
	screen.blit(background, (0, 0))
	pygame.display.flip()
	
	hits = []
	gameover = 0
	animation = 0
	currentminute = 0

	while ((float(currentminute)/60.0) < minutedelay):
		for event in pygame.event.get():
			if event.type == pygame.QUIT:
				print "Exiting...."
				sys.exit()
			if event.type == pygame.KEYDOWN:
				if event.key == pygame.K_KP_PLUS:
					speed = speed + 1
					print "Changed scope speed to " + str(speed)
				if event.key == pygame.K_KP_MINUS:
					speed = speed - 1
					print "Changed scope speed to " + str(speed)
		 
		screen.blit(background, (0, 0))
		
		for hit in hits[0:-1]:
			text = font.render(str(hit), 1, pykino_colors.green1)
			textpos = text.get_rect()
			textpos.centerx = calcx(hit-1)
			textpos.centery = calcy(hit-1)
			screen.blit(text, textpos)
		
		if (not animation):
			if (hits.__len__()<20):
				newhit = 0
				while ((newhit==0) or (hits.count(newhit)!=0)):
					newhit = random.randint(1,80)
				
				print "And the new number is...." + str(newhit)
				hits.append(newhit)
				animation = 1
			else:
				# the last number should be highlighted as well
				text = font.render(str(hits[-1]), 1, pykino_colors.green1)
				textpos = text.get_rect()
				textpos.centerx = calcx(hits[-1]-1)
				textpos.centery = calcy(hits[-1]-1)
				screen.blit(text, textpos)
				
				# game over message
				color = pykino_colors.red
				t = int(minutedelay*60.0 - currentminute)
				m = t / 60
				s = t % 60
				tt = "%d:%02d" % (m, s)
				text = font.render("Next game in " + tt + " minutes", 1, color)
				textpos = text.get_rect()
				textpos.centerx = background.get_rect().centerx
				textpos.centery = 567
				screen.blit(text, textpos)
				pygame.display.flip()
				pygame.time.delay(1000-40) # see end of the loop for explanation
				currentminute = currentminute + 1
		
		else:
			try:
				f = hits[-2]
			except IndexError:
				f = 1
			t = hits[-1]
			
			dx = float(calcx(t-1) - calcx(f-1))
			dy = float(calcy(t-1) - calcy(f-1))
			dp = math.sqrt(dx**2 + dy**2)

			cp = float(speed * animation)

			if (cp > dp): # we are done
				animation = 0
				if (sounds):
					pykino_utils.playmusic("blip.wav",0.5);
			else:
				animation = animation + 1
				sind = dy/dp
				cosd = dx/dp
				cx = cp*cosd
				cy = cp*sind
				
				(scope_image, scope_rect) = pykino_utils.load_png('data/image/scope.png')
				scope_rect.centerx = calcx(f-1)+cx
				scope_rect.centery = calcy(f-1)+cy
				screen.blit(scope_image, scope_rect)
				
		pygame.display.flip()
		pygame.time.delay(40)
	return speed

def yesno(i):
	if (i):
		return "yes"
	return "no"

def print_config():
	print "Configuration:"
	print "\tFullscreen:\t\t" + yesno(config.fullscreen)
	print "\tResolution:\t\t" + str(config.width) + "x" + str(config.height)
	print "\tSpeed of the scope:\t" + str(config.speed)
	print "\tDelay between games:\t" + str(config.delay) + " (minutes)"
	print "\tSounds enabled:\t\t" + yesno(config.sound)

if (__name__ == '__main__'):
	main()
